﻿using System;
using System.ComponentModel;
using System.Reflection;
using Wisej.Core;
using Wisej.Web;

namespace WisejWebApplication
{
    public class VirtualDataGridView : Wisej.Web.DataGridView
    {
        [Browsable(false)]
        public int LazyTimeout { get; set; }
        public VirtualDataGridView()
        {
            VirtualMode = false;
        }
        public bool UseInternalPaging { get { return true; } set { } }
        public string CustomStyle { get { return String.Empty; } set { } }
        //public int BorderWidth { get; set; }
        public int VirtualBlockSize { get { return 10; } set { } }
        public int FirstDisplayedScrollingRowIndex { get { return FirstDisplayedRowIndex; } set { } }
        public void RefreshEdit() { }

        //public override void Sort(DataGridViewColumn column, ListSortDirection direction)
        //{
        //    var DataConnection = this.GetPropertyValue("DataConnection");
        //    CurrencyManager currencyManager = DataConnection.GetPropertyValue("CurrencyManager") as CurrencyManager;
        //    IBindingList list = DataConnection.GetPropertyValue("List") as IBindingList;
        //    if (list.SupportsSorting)
        //    {

        //    }
        //    base.Sort(column, direction);
        //}

        protected override void OnControlCreated(EventArgs e)
        {
            if (!DesignMode && LazyTimeout > 0)
            {
                // attach javascript event to defer the selection change.
                AddClientEventListener("selectionChanged",
                @"

			var me = this, data = e.getData();
			clearTimeout(this._selectionChangedTimer);
			this._selectionChangedTimer = setTimeout(function() {
				me.fireDataEvent('lazySelectionChanged', data);
			}, " + LazyTimeout + @");

			");

                // attach javascript event to defer the selection change.
                AddClientEventListener("focusCellChanged",
                @"

			var me = this, data = e.getData();
			clearTimeout(this._focusCellChanged);
			this._focusCellChangedTimer = setTimeout(function() {
				me.fireDataEvent('lazyfocusCellChanged', data);
			}, " + LazyTimeout + @");

			");
            }
            base.OnControlCreated(e);
        }

        protected override void OnWebEvent(WisejEventArgs e)
        {
            switch (e.Type)
            {
                case "lazySelectionChanged":
                    {
                        var args = new WisejEventArgs(this, "selectionChanged", new DynamicObject());
                        args.Parameters.Data = e.Parameters.Data;
                        base.OnWebEvent(args);
                    }
                    break;

                case "lazyFocusCellChanged":
                    {
                        var args = new WisejEventArgs(this, "focusCellChanged", new DynamicObject());
                        args.Parameters.Cell = e.Parameters.Cell;
                        base.OnWebEvent(args);
                    }
                    break;

                default:
                    base.OnWebEvent(e);
                    break;
            }
        }

        protected override void OnWebRender(dynamic config)
        {
            base.OnWebRender((object)config);
            if (LazyTimeout > 0)
            {
                // detach the live event.
                config.wiredEvents.Remove("selectionChanged(Data)");
                config.wiredEvents.Remove("focusCellChanged(Cell)");

                // attach deferred event.
                config.wiredEvents.Add("lazySelectionChanged(Data)", "lazyFocusCellChanged(Cell)");
            }
        }
    }

    //public static class ReflectionHelper
    //{
    //    private static PropertyInfo GetPropertyInfo(Type type, string propertyName)
    //    {
    //        PropertyInfo propInfo = null;
    //        do
    //        {
    //            propInfo = type.GetProperty(propertyName,
    //                   BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic);
    //            type = type.BaseType;
    //        }
    //        while (propInfo == null && type != null);
    //        return propInfo;
    //    }

    //    public static object GetPropertyValue(this object obj, string propertyName)
    //    {
    //        if (obj == null)
    //            throw new ArgumentNullException("obj");
    //        Type objType = obj.GetType();
    //        PropertyInfo propInfo = GetPropertyInfo(objType, propertyName);
    //        if (propInfo == null)
    //            throw new ArgumentOutOfRangeException("propertyName",
    //              string.Format("Couldn't find property {0} in type {1}", propertyName, objType.FullName));
    //        return propInfo.GetValue(obj, null);
    //    }

    //    public static void SetPropertyValue(this object obj, string propertyName, object val)
    //    {
    //        if (obj == null)
    //            throw new ArgumentNullException("obj");
    //        Type objType = obj.GetType();
    //        PropertyInfo propInfo = GetPropertyInfo(objType, propertyName);
    //        if (propInfo == null)
    //            throw new ArgumentOutOfRangeException("propertyName",
    //              string.Format("Couldn't find property {0} in type {1}", propertyName, objType.FullName));
    //        propInfo.SetValue(obj, val, null);
    //    }
    //}

}
